﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Livet;
using Livet.Commands;
using Livet.Messaging;
using Livet.Messaging.IO;
using Livet.Messaging.Windows;

using LivetSampleProject.Models;
using System.Collections.ObjectModel;

namespace LivetSampleProject.ViewModels
{
    public class MainWindowViewModel : ViewModel
    {
        /*コマンド、プロパティの定義にはそれぞれ 
         * 
         *  lvcom   : ViewModelCommand
         *  llcom   : ListenerCommand
         *  lvcomn  : ViewModelCommand(CanExecute無し)
         *  llcomn  : ListenerCommand(CanExecute無し)
         *  lprop   : 変更通知プロパティ
         *  
         * を使用してください。
         */
        
        

        /*ViewModelからViewを操作したい場合は、
         * Messengerプロパティからメッセージ(各種InteractionMessage)を発信してください。
         */

        /*
         * UIDispatcherを操作する場合は、DispatcherHelperのメソッドを操作してください。
         * UIDispatcher自体はApp.xaml.csでインスタンスを確保してあります。
         */

        Main _model;

        public MainWindowViewModel()
        {
            _model = new Main();

            //変更通知をDispatcher経由で行うコレクションを、Modelのコレクションから生成しています。
            //作成されたコレクションは、ソースのコレクションと同期されます。
            //弱イベントを使用してリークの心配のない方法で同期されています。
            Members = ViewModelHelper.CreateReadOnlyNotificationDispatcherCollection(
                _model.Members,
                m => new MemberViewModel(m,this),
                DispatcherHelper.UIDispatcher);

            CheckedViewModel = new ObservableCollection<MemberViewModel>();

            CheckedViewModel.CollectionChanged += (sender, e) => this.RemoveCommand.RaiseCanExecuteChanged();
        }

        public ReadOnlyNotificationDispatcherCollection<MemberViewModel> Members
        {
            get;
            private set;
        }

        internal ObservableCollection<MemberViewModel> CheckedViewModel
        {
            get;
            private set;
        }

        #region EditNewCommand
        ViewModelCommand _EditNewCommand;

        public ViewModelCommand EditNewCommand
        {
            get
            {
                if (_EditNewCommand == null)
                    _EditNewCommand = new ViewModelCommand(EditNew);
                return _EditNewCommand;
            }
        }

        private void EditNew()
        {
            //Viewに画面遷移用メッセージを送信しています。
            //Viewは対応するメッセージキーを持つInteractionTransitionMessageTriggerでこのメッセージを受信します。
            Messenger.Raise(new TransitionMessage(new MemberViewModel(new Member(_model),this), "Transition"));
        }
        #endregion

        
        #region RemoveCommand
        ListenerCommand<ConfirmationMessage> _RemoveCommand;

        public ListenerCommand<ConfirmationMessage> RemoveCommand
        {
            get
            {
                if (_RemoveCommand == null)
                    _RemoveCommand = new ListenerCommand<ConfirmationMessage>(Remove, CanRemove);
                return _RemoveCommand;
            }
        }

        private bool CanRemove()
        {
            return CheckedViewModel.Any();
        }

        private void Remove(ConfirmationMessage message)
        {
            if (message.Response.HasValue && message.Response.Value)
            {
                CheckedViewModel.ToList().ForEach(m => m.RemoveCommand.Execute());
            }
        }
        #endregion
    }
}
